@@ -16,6 +16,7 @@ from simple_ado.context import ADOContext
 from simple_ado.exceptions import ADOException
 from simple_ado.git import ADOGitClient
 from simple_ado.graph import ADOGraphClient
+from simple_ado.governance import ADOGovernanceClient
 from simple_ado.http_client import ADOHTTPClient, ADOResponse
 from simple_ado.pools import ADOPoolsClient
 from simple_ado.pull_requests import ADOPullRequestClient
@@ -41,11 +42,12 @@ class ADOClient:
 
     log: logging.Logger
 
-    _context: ADOContext
+    context: ADOContext
     http_client: ADOHTTPClient
 
     builds: ADOBuildClient
     git: ADOGitClient
+    governance: ADOGovernanceClient
     graph: ADOGraphClient
     pools: ADOPoolsClient
     security: ADOSecurityClient
@@ -71,7 +73,7 @@ class ADOClient:
         else:
             self.log = log.getChild("ado")
 
-        self._context = ADOContext(
+        self.context = ADOContext(
             username=username, repository_id=repository_id, status_context=status_context
         )
 
@@ -83,13 +85,14 @@ class ADOClient:
             extra_headers=extra_headers,
         )
 
-        self.builds = ADOBuildClient(self._context, self.http_client, self.log)
-        self.git = ADOGitClient(self._context, self.http_client, self.log)
-        self.graph = ADOGraphClient(self._context, self.http_client, self.log)
-        self.pools = ADOPoolsClient(self._context, self.http_client, self.log)
-        self.security = ADOSecurityClient(self._context, self.http_client, self.log)
-        self.user = ADOUserClient(self._context, self.http_client, self.log)
-        self.workitems = ADOWorkItemsClient(self._context, self.http_client, self.log)
+        self.builds = ADOBuildClient(self.context, self.http_client, self.log)
+        self.git = ADOGitClient(self.context, self.http_client, self.log)
+        self.governance = ADOGovernanceClient(self.context, self.http_client, self.log)
+        self.graph = ADOGraphClient(self.context, self.http_client, self.log)
+        self.pools = ADOPoolsClient(self.context, self.http_client, self.log)
+        self.security = ADOSecurityClient(self.context, self.http_client, self.log)
+        self.user = ADOUserClient(self.context, self.http_client, self.log)
+        self.workitems = ADOWorkItemsClient(self.context, self.http_client, self.log)
 
     def verify_access(self) -> bool:
         """Verify that we have access to ADO.
@@ -97,7 +100,7 @@ class ADOClient:
         :returns: True if we have access, False otherwise
         """
 
-        request_url = f"{self.http_client.base_url()}/git/repositories?api-version=1.0"
+        request_url = f"{self.http_client.api_endpoint()}/git/repositories?api-version=1.0"
 
         try:
             response = self.http_client.get(request_url)
@@ -116,7 +119,7 @@ class ADOClient:
         title: Optional[str] = None,
         description: Optional[str] = None,
         reviewer_ids: Optional[List[str]] = None,
-    ) -> ADOPullRequestClient:
+    ) -> ADOResponse:
         """Creates a pull request with the given information
 
         :param source_branch: The source branch of the pull request
@@ -131,7 +134,9 @@ class ADOClient:
         """
         self.log.debug("Creating pull request")
 
-        request_url = f"{self.http_client.base_url()}/git/repositories/{self._context.repository_id}"
+        request_url = (
+            f"{self.http_client.api_endpoint()}/git/repositories/{self.context.repository_id}"
+        )
         request_url += "/pullRequests?api-version=5.1"
 
         body: Dict[str, Any] = {
@@ -158,7 +163,7 @@ class ADOClient:
 
         :returns: A new ADOPullRequest client for the pull request specified
         """
-        return ADOPullRequestClient(self._context, self.http_client, self.log, pull_request_id)
+        return ADOPullRequestClient(self.context, self.http_client, self.log, pull_request_id)
 
     def list_all_pull_requests(self, *, branch_name: Optional[str] = None) -> ADOResponse:
         """Get the pull requests for a branch from ADO.
@@ -176,7 +181,8 @@ class ADOClient:
 
         while True:
 
-            request_url = f"{self.http_client.base_url()}/git/repositories/{self._context.repository_id}/pullRequests?"
+            request_url = f"{self.http_client.api_endpoint()}/git/repositories"
+            request_url += f"/{self.context.repository_id}/pullRequests?"
 
             request_url += f"$top=100&$skip={offset}"
 
@@ -206,7 +212,7 @@ class ADOClient:
         parameters: Dict[str, Any],
         is_default_collection: bool = True,
         is_project: bool = True,
-        is_internal: bool = False
+        is_internal: bool = False,
     ) -> ADOResponse:
         """Perform a custom GET REST request.
 
@@ -229,15 +235,16 @@ class ADOClient:
         """
 
         encoded_parameters = urllib.parse.urlencode(parameters)
-        request_url = self.http_client.base_url(
+        request_url = self.http_client.api_endpoint(
             is_default_collection=is_default_collection,
             is_project=is_project,
-            is_internal=is_internal
+            is_internal=is_internal,
         )
         request_url += f"/{url_fragment}?{encoded_parameters}"
 
         return self.http_client.get(request_url)
 
+
 def _canonicalize_branch_name(branch_name: str) -> str:
     """Cleanup the branch name before sending it via ADO request
 
