@@ -8,6 +8,7 @@
 import logging
 from typing import Any, Dict, List, Optional
 
+import deserialize
 import requests
 
 from simple_ado.base_client import ADOBaseClient
@@ -22,6 +23,8 @@ from simple_ado.exceptions import ADOException
 from simple_ado.git import ADOGitStatusState
 from simple_ado.http_client import ADOHTTPClient, ADOResponse, ADOThread
 
+from simple_ado.models import PatchOperation, AddOperation, DeleteOperation, PropertyValue
+
 
 class ADOPullRequestClient(ADOBaseClient):
     """Wrapper class around the ADO Pull Request APIs.
@@ -51,7 +54,9 @@ class ADOPullRequestClient(ADOBaseClient):
         """
 
         self.log.debug(f"Getting PR: {self.pull_request_id}")
-        request_url = f"{self.http_client.base_url()}/git/repositories/{self._context.repository_id}"
+        request_url = (
+            f"{self.http_client.api_endpoint()}/git/repositories/{self.context.repository_id}"
+        )
         request_url += f"/pullRequests/{self.pull_request_id}?api-version=3.0-preview"
         response = self.http_client.get(request_url)
         return self.http_client.decode_response(response)
@@ -63,7 +68,9 @@ class ADOPullRequestClient(ADOBaseClient):
         """
 
         self.log.debug(f"Getting workitems: {self.pull_request_id}")
-        request_url = f"{self.http_client.base_url()}/git/repositories/{self._context.repository_id}"
+        request_url = (
+            f"{self.http_client.api_endpoint()}/git/repositories/{self.context.repository_id}"
+        )
         request_url += f"/pullRequests/{self.pull_request_id}/workitems?api-version=5.0"
         response = self.http_client.get(request_url)
         return self.http_client.decode_response(response)
@@ -77,7 +84,9 @@ class ADOPullRequestClient(ADOBaseClient):
         """
 
         self.log.debug(f"Getting threads: {self.pull_request_id}")
-        request_url = f"{self.http_client.base_url()}/git/repositories/{self._context.repository_id}"
+        request_url = (
+            f"{self.http_client.api_endpoint()}/git/repositories/{self.context.repository_id}"
+        )
         request_url += f"/pullRequests/{self.pull_request_id}/threads?api-version=3.0-preview"
         response = self.http_client.get(request_url)
         response_data = self.http_client.decode_response(response)
@@ -116,7 +125,7 @@ class ADOPullRequestClient(ADOBaseClient):
         comment: ADOComment,
         *,
         status: Optional[ADOCommentStatus] = None,
-        comment_identifier: Optional[str] = None
+        comment_identifier: Optional[str] = None,
     ) -> ADOResponse:
         """Create a thread using a single root comment.
 
@@ -133,7 +142,7 @@ class ADOPullRequestClient(ADOBaseClient):
             [comment.generate_representation()],
             thread_location=comment.location,
             status=status,
-            comment_identifier=comment_identifier
+            comment_identifier=comment_identifier,
         )
 
     def create_thread(
@@ -157,7 +166,9 @@ class ADOPullRequestClient(ADOBaseClient):
 
         self.log.debug(f"Creating thread ({self.pull_request_id})")
 
-        request_url = f"{self.http_client.base_url()}/git/repositories/{self._context.repository_id}"
+        request_url = (
+            f"{self.http_client.api_endpoint()}/git/repositories/{self.context.repository_id}"
+        )
         request_url += f"/pullRequests/{self.pull_request_id}/threads?api-version=3.0-preview"
 
         properties = {
@@ -165,7 +176,9 @@ class ADOPullRequestClient(ADOBaseClient):
         }
 
         if comment_identifier:
-            properties[ADOCommentProperty.COMMENT_IDENTIFIER] = ADOCommentProperty.create_string(comment_identifier)
+            properties[ADOCommentProperty.COMMENT_IDENTIFIER] = ADOCommentProperty.create_string(
+                comment_identifier
+            )
 
         body = {
             "comments": comments,
@@ -193,7 +206,7 @@ class ADOPullRequestClient(ADOBaseClient):
             comment_id = comment["id"]
             self.log.debug(f"Deleting comment: {comment_id}")
             request_url = (
-                f"{self.http_client.base_url()}/git/repositories/{self._context.repository_id}"
+                f"{self.http_client.api_endpoint()}/git/repositories/{self.context.repository_id}"
             )
             request_url += f"/pullRequests/{self.pull_request_id}/threads/{thread_id}"
             request_url += f"/comments/{comment_id}?api-version=3.0-preview"
@@ -203,7 +216,9 @@ class ADOPullRequestClient(ADOBaseClient):
                 headers=self.http_client.construct_headers(),
             )
 
-    def create_thread_list(self, threads: List[ADOComment], comment_identifier: Optional[str] = None) -> None:
+    def create_thread_list(
+        self, threads: List[ADOComment], comment_identifier: Optional[str] = None
+    ) -> None:
         """Create a list of threads
 
         :param List[ADOComment] threads: The threads to create
@@ -248,13 +263,15 @@ class ADOPullRequestClient(ADOBaseClient):
             f"Setting PR status ({state}) on PR ({self.pull_request_id}): {identifier} -> {description}"
         )
 
-        request_url = f"{self.http_client.base_url()}/git/repositories/{self._context.repository_id}"
+        request_url = (
+            f"{self.http_client.api_endpoint()}/git/repositories/{self.context.repository_id}"
+        )
         request_url += f"/pullRequests/{self.pull_request_id}/statuses?api-version=4.0-preview"
 
         body = {
             "state": state.value,
             "description": description,
-            "context": {"name": self._context.status_context, "genre": identifier},
+            "context": {"name": self.context.status_context, "genre": identifier},
         }
 
         if iteration is not None:
@@ -288,7 +305,7 @@ class ADOPullRequestClient(ADOBaseClient):
 
         try:
             # It wasn't one of the specified users comments
-            if thread["comments"][0]["author"]["uniqueName"] != self._context.username:
+            if thread["comments"][0]["author"]["uniqueName"] != self.context.username:
                 return False
         except:
             raise ADOException(
@@ -352,3 +369,71 @@ class ADOPullRequestClient(ADOBaseClient):
         for thread in self.threads_with_identifier(identifier):
             self.log.debug(f"Deleting thread: {thread}")
             self.delete_thread(thread)
+
+    def get_properties(self) -> Dict[str, PropertyValue]:
+        """Get the properties on the PR from ADO.
+
+        :returns: The properties that were found
+        """
+
+        self.log.debug(f"Getting properties: {self.pull_request_id}")
+        request_url = (
+            f"{self.http_client.api_endpoint()}/git/repositories/{self.context.repository_id}"
+        )
+        request_url += f"/pullRequests/{self.pull_request_id}/properties?api-version=5.1-preview.1"
+        response = self.http_client.get(request_url)
+        response_data = self.http_client.decode_response(response)
+        raw_properties = self.http_client.extract_value(response_data)
+
+        properties = deserialize.deserialize(Dict[str, PropertyValue], raw_properties)
+
+        return properties
+
+    def patch_properties(self, operations: List[PatchOperation]) -> Dict[str, PropertyValue]:
+        """Patch the properties on the PR.
+
+        Usually add_property(), delete_property() and update_property() are
+        going to be what you need instead of this base function.
+
+        :param operations: The raw operations
+
+        :returns: The new properties
+        """
+
+        self.log.debug(f"Patching properties: {self.pull_request_id}")
+        request_url = (
+            f"{self.http_client.api_endpoint()}/git/repositories/{self.context.repository_id}"
+        )
+        request_url += f"/pullRequests/{self.pull_request_id}/properties?api-version=5.1-preview.1"
+
+        response = self.http_client.patch(request_url, operations=operations)
+
+        response_data = self.http_client.decode_response(response)
+        raw_properties = self.http_client.extract_value(response_data)
+
+        properties = deserialize.deserialize(Dict[str, PropertyValue], raw_properties)
+
+        return properties
+
+    def add_property(self, name: str, value: str) -> Dict[str, PropertyValue]:
+        """Add a property to the PR.
+
+        :param name: The name of the property to add
+        :param value: The value of the property to add
+
+        :returns: The new properties
+        """
+
+        operation = AddOperation("/" + name, value)
+        return self.patch_properties([operation])
+
+    def delete_property(self, name: str) -> Dict[str, PropertyValue]:
+        """Delete a property from the PR.
+
+        :param name: The name of the property to delete
+
+        :returns: The new properties
+        """
+
+        operation = DeleteOperation("/" + name)
+        return self.patch_properties([operation])
